import pandas as pd
import numpy as np
import matplotlib.cm as cm
import matplotlib.pyplot as plt
import networkx as nx
import os
import json
import fetch_csv_from_compass
import postResults
import time
# from community import community_louvain
from cdlib import algorithms

DATA_DIR = '/data'
OUTPUT_DIR = '/data'

def run(source):
    COMPASS_CALLBACK_URL = os.getenv("PORTFOLIO_API_URL")

    print ('====> Fetch Data From Compass')
    fetch_csv_from_compass.fetchData(source)
    print ('====> Fetch Data From Compass: Done')
    def community_layout(g, partition):
        
        pos_communities = _position_communities(g, partition, scale=3.0)

        pos_nodes = _position_nodes(g, partition, scale=2.)

        # combine positions
        pos = dict()
        for node in g.nodes():
            pos[node] = pos_communities[node]  + pos_nodes[node]

        return pos

    def _position_communities(g, partition, **kwargs):

        between_community_edges = _find_between_community_edges(g, partition)

        communities = set(partition.values())
        hypergraph = nx.DiGraph()
        hypergraph.add_nodes_from(communities)
        for (ci, cj), edges in between_community_edges.items():
            hypergraph.add_edge(ci, cj, weight=len(edges))

        pos_communities = nx.spring_layout(hypergraph, **kwargs)

        pos = dict()
        for node, community in partition.items():
            pos[node] = pos_communities[community]

        return pos

    def _find_between_community_edges(g, partition):

        edges = dict()

        for (ni, nj) in g.edges():
            ci = partition[ni]
            cj = partition[nj]

            if ci != cj:
                try:
                    edges[(ci, cj)] += [(ni, nj)]
                except KeyError:
                    edges[(ci, cj)] = [(ni, nj)]

        return edges

    def _position_nodes(g, partition, **kwargs):
        
        communities = dict()
        for node, community in partition.items():
            try:
                communities[community] += [node]
            except KeyError:
                communities[community] = [node]

        pos = dict()
        for ci, nodes in communities.items():
            subgraph = g.subgraph(nodes)
            pos_subgraph = nx.spring_layout(subgraph, **kwargs)
            pos.update(pos_subgraph)

        return pos

    def save_communities_to_json_and_post(g, coms, fileName):
         
        move_groups = {"name" : "Move Groups", "children": []}
        index = 0
        for com in coms.communities:
            index = index + 1
            move_group = {"name" : str(index), "children": []}
            for node in com:
                move_group["children"].append({"name": node})
            move_groups["children"].append(move_group)
        
        mv_json = json.dumps(move_groups)
        #print(mv_json)
        fin = open("move-group.html", "rt")
        #output file to write the result to
        ts = time.time()
        fullFileName = "/data/move-group-"+fileName+"-"+str(ts)+".html"

        fout = open(fullFileName, "wt")
        #for each line in the input file
        for line in fin:
            #read replace the string and write to output file
            fout.write(line.replace('<<json>>', mv_json))
        #close input and output files
        fin.close()
        fout.close()
        print ('====> Post Results')
        postResults.postFile(f"{COMPASS_CALLBACK_URL}/artifact",fullFileName, "Move Groups based on VM to VM Traffic - ["+fileName+"]", "Move Group Visualization using ["+fileName+" "+source+"]", "Move Group Python Plugin", "VRNI" )  
        print ('====> Post Results: Done!') 


    
    def save_communities_to_json_and_csv_and_post(g, coms, fileName):
             
        move_groups = {"name" : "Move Groups", "children": []}
        index = 0
        for com in coms.communities:
            index = index + 1
            move_group = {"name" : str(index), "children": []}
            for node in com:
                move_group["children"].append({"name": node})
            move_groups["children"].append(move_group)

        mv_json = json.dumps(move_groups)
        mv_json = json.loads("["+mv_json+"]")
        print(mv_json)
        ts = time.time()
        fullFileName = "/data/move-group-"+fileName+"-"+str(ts)+".csv"
        csvFile = open(fullFileName, 'w')
        csvFile.write("Name,Group\n")

        for item in mv_json:
            for child in item['children']:
                print("GROUP="+child['name'])
                for vm in child['children']:
                    print("-----VM="+vm['name'])
                    csvFile.write(child['name']+","+vm['name']+"\n")
        csvFile.close()

        print ('====> Post Results')
        postResults.postFile(f"{COMPASS_CALLBACK_URL}/artifact",fullFileName, "Move Groups Details [CSV] - ["+fileName+"]", "Move Group CSV Breakdown Details using ["+fileName+" "+source+"]", "Move Group Python Plugin", "VRNI" ) 
        print ('====> Post Results: Done!') 
            
        dfFlows = pd.read_csv('/data/data.csv')
    dfSourceApps = dfFlows[['Source_Application']]
    dfSourceApps.rename(columns={'Source_Application': 'Application'}, inplace=True)
    dfDestApps = dfFlows[['Destination_Application']]
    dfDestApps.rename(columns={'Destination_Application': 'Application'}, inplace=True)
    dfAllApps = pd.concat([dfSourceApps, dfDestApps])
    appList = dfAllApps['Application'].drop_duplicates().to_list()
        
    print ('====> Leiden: Start!')
    g = nx.from_pandas_edgelist(dfFlows, source='Source_Application', target='Destination_Application', edge_attr=True)
    coms = algorithms.leiden(g)
    print ('====> Leiden: Done!')
    save_communities_to_json_and_post(g, coms, "leiden")
    save_communities_to_json_and_csv_and_post(g, coms, "leiden")